import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;

import org.openqa.selenium.By;
import org.openqa.selenium.NoSuchElementException;
import org.openqa.selenium.WebDriver;
import org.openqa.selenium.WebElement;
import org.openqa.selenium.chrome.ChromeDriver;
import org.openqa.selenium.firefox.FirefoxDriver;
import org.openqa.selenium.remote.CapabilityType;
import org.openqa.selenium.remote.DesiredCapabilities;


public class UrlFileProcessor 
{
	
	/**
	 * @param args
	 * @throws InterruptedException 
	 * @throws IOException 
	 */
	
	//NOTE: This main method exists for testing purposes
	//It will not be instantiated in the automated process
	public static void main(String[] args) throws InterruptedException
	{
		if(args.length < 1)
		{
			System.err.println("Invalid arguments given to URL File Processor");
		}
		else
		{			
			//Create Selennium driver object with the proxy configuration enabled
			WebDriver driver = new ChromeDriver();
			
			try
			{
				processUrlFile(args[0], driver);
			}
			catch(IOException e)
			{
				e.printStackTrace();
			}
			driver.close();
		}
	}
	
	//quits and returns a new webdriver
	private static WebDriver resetDriver(WebDriver d)
	{
		d.quit();
	    org.openqa.selenium.Proxy proxy = new org.openqa.selenium.Proxy();
	    proxy.setHttpProxy("localhost:8080")
	     	.setFtpProxy("localhost:8080")
	     	.setSslProxy("localhost:8080");
		   //Proxy Configuration
		   DesiredCapabilities cap = new DesiredCapabilities();
		   cap.setCapability(CapabilityType.PROXY, proxy);
		
		   WebDriver driver= new FirefoxDriver(cap);
		   driver.manage().timeouts().pageLoadTimeout(60, java.util.concurrent.TimeUnit.SECONDS);
		   return driver;
	}
	
	//runs commands
	public static WebDriver processUrlFile(String path, WebDriver driver) throws IOException, InterruptedException{
		BufferedReader in = new BufferedReader(new FileReader(path));	

		String line = null;  
		while ((line = in.readLine()) != null)  
		{  
		   if(line.startsWith("\t")) processCommand(line, driver);
		   else 
		   {
				 driver = goToUrl(line, driver);
		   }
		}
		
		in.close();
		System.out.println("Finished with lines, waiting");
		
		try 
		{
		Thread.sleep(5000);
		}
		catch(Exception e)
		{
			System.out.println("Exception in sleep");
			return driver;
		}
		return driver;
	}

	//navigates to a given url with the input driver
	private static WebDriver goToUrl(String url, WebDriver driver) throws InterruptedException {
		try
		{
			driver.get(url);
		}
		catch(org.openqa.selenium.TimeoutException e)
		{
			System.out.println("Page timeout");
			driver=resetDriver(driver);
			return driver;
		}
		catch( org.openqa.selenium.WebDriverException e) 
		{
			driver=resetDriver(driver);
			return driver;
		}
		
		try
		{
			Thread.sleep(3000);
		}
		catch(InterruptedException e)
		{
			System.err.println("Exception in sleep");
			e.printStackTrace();
			return driver;
		}
		
		return driver;
	}

	//command processor
	//this is where you can add in new script command support
	private static void processCommand(String line, WebDriver driver) 
	{
		String[] command = line.substring(1).split("\\|");//remove leading tab and split on pipes		
		//System.out.println(command[0]);
		if(command[0].equals("TYPE"))
		{
			WebElement element = getElementByNameOrID(command[1], driver);
			element.clear();
			element.sendKeys(command[2]);
		}
		else if(command[0].equals("BUTTON"))
		{
			By by = By.xpath("//input[@value='" +command[1]+"']");
			driver.findElement(by).click();
		}
		else if(command[0].equals("WAIT"))
		{
			try 
			{
				long length = Long.parseLong(command[1]);
				//System.out.println(length);
				Thread.sleep(length);
			} 
			catch (NumberFormatException e) 
			{
				System.err.println("Invalid value for sleep command, continuing");
				e.printStackTrace();
			} 
			catch (InterruptedException e) 
			{
				e.printStackTrace();
			}
		}
	}
	
	//finds an element on a given page
	private static WebElement getElementByNameOrID(String tofind, WebDriver driver)
	{
		for(int i=0; i<40; i++)
		{
			try
			{
				return driver.findElement(By.name(tofind));
			}
			catch(NoSuchElementException e)
			{
				try
				{
					return driver.findElement(By.id(tofind));
				}
				catch(NoSuchElementException f)
				{
					//no-op, if element is not found just continue on to the next one
				}
			}
			try 
			{
				Thread.sleep(100);
			} catch (InterruptedException e) 
			{
				Thread.currentThread().interrupt(); //restore interrupted status
			}
		}
		throw new NoSuchElementException("name or id: "+tofind);
	}
}

